#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphFactory
from pyams_content.component.paragraph.interfaces.header import IHeaderParagraph, HEADER_PARAGRAPH_TYPE, \
    HEADER_PARAGRAPH_RENDERERS
from pyams_content.features.checker.interfaces import IContentChecker, MISSING_VALUE, MISSING_LANG_VALUE
from pyams_content.features.renderer.interfaces import IContentRenderer
from pyams_i18n.interfaces import II18n, II18nManager, INegotiator

# import packages
from pyams_content.component.paragraph import BaseParagraph, BaseParagraphContentChecker, BaseParagraphFactory
from pyams_content.features.renderer import RenderedContentMixin
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import utility_config, get_utility
from pyams_utils.request import check_request
from pyams_utils.text import get_text_start
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from pyams_content import _


@implementer(IHeaderParagraph)
class HeaderParagraph(RenderedContentMixin, BaseParagraph):
    """Header paragraph"""

    icon_class = 'fa-header'
    icon_hint = _("Header")

    @property
    def title(self):
        header = II18n(self).query_attribute('header')
        return get_text_start(header, 50, 10)

    header = FieldProperty(IHeaderParagraph['header'])
    renderer = FieldProperty(IHeaderParagraph['renderer'])


@utility_config(name=HEADER_PARAGRAPH_TYPE, provides=IParagraphFactory)
class HeaderParagraphFactory(BaseParagraphFactory):
    """Header paragraph factory"""

    name = _("Header paragraph")
    content_type = HeaderParagraph


@adapter_config(context=IHeaderParagraph, provides=IContentChecker)
class HeaderParagraphContentChecker(BaseParagraphContentChecker):
    """Header paragraph content checker"""

    def inner_check(self, request):
        output = []
        translate = request.localizer.translate
        manager = get_parent(self.context, II18nManager)
        if manager is not None:
            langs = manager.get_languages()
        else:
            negotiator = get_utility(INegotiator)
            langs = (negotiator.server_language, )
        i18n = II18n(self.context)
        for lang in langs:
            value = i18n.get_attribute('header', lang, request)
            if not value:
                field_title = translate(IHeaderParagraph['header'].title)
                if len(langs) == 1:
                    output.append(translate(MISSING_VALUE).format(field=field_title))
                else:
                    output.append(translate(MISSING_LANG_VALUE).format(field=field_title, lang=lang))
        return output


@vocabulary_config(name=HEADER_PARAGRAPH_RENDERERS)
class HeaderParagraphRendererVocabulary(SimpleVocabulary):
    """Header paragraph renderers vocabulary"""

    def __init__(self, context=None):
        request = check_request()
        translate = request.localizer.translate
        registry = request.registry
        if not IHeaderParagraph.providedBy(context):
            context = HeaderParagraph()
        terms = [SimpleTerm(name, title=translate(adapter.label))
                 for name, adapter in sorted(registry.getAdapters((context, request), IContentRenderer),
                                             key=lambda x: x[1].weight)]
        super(HeaderParagraphRendererVocabulary, self).__init__(terms)
