#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import json

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    IParagraphRenderer
from pyams_content.component.paragraph.interfaces.milestone import MILESTONE_PARAGRAPH_TYPE, IMilestoneParagraph, \
    IMilestoneContainer, IMilestoneContainerTarget, IMilestone
from pyams_content.component.paragraph.zmi import IParagraphContainerView
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_content.shared.common import IWfSharedContent
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons, IInnerSubForm
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu, IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import MANAGE_PERMISSION
from z3c.form.interfaces import INPUT_MODE
from z3c.table.interfaces import IValues, IColumn

# import packages
from pyams_content.component.paragraph.milestone import MilestoneParagraph, Milestone
from pyams_content.component.paragraph.zmi import BaseParagraphAddMenu, BaseParagraphAJAXAddForm, \
    BaseParagraphPropertiesEditForm, BaseParagraphAJAXEditForm
from pyams_content.features.renderer.zmi import BaseRenderedContentRenderer
from pyams_content.features.renderer.zmi.widget import RendererFieldWidget
from pyams_content.skin import pyams_content
from pyams_form.form import AJAXAddForm, AJAXEditForm
from pyams_form.security import ProtectedFormObjectMixin
from pyams_i18n.column import I18nAttrColumn
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.table import BaseTable, SorterColumn, JsActionColumn, I18nColumn, TrashColumn
from pyams_skin.viewlet.toolbar import ToolbarAction
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.fanstatic import get_resource_path
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, InnerAdminDisplayForm, AdminDialogEditForm
from pyramid.decorator import reify
from pyramid.exceptions import NotFound
from pyramid.view import view_config
from z3c.form import field, button
from z3c.table.column import GetAttrColumn
from zope.interface import implementer, Interface

from pyams_content import _


class IMilestonesView(Interface):
    """Milestones view marker interface"""


class IMilestonesParentForm(Interface):
    """Milestones parent form marker interface"""


@viewlet_config(name='add-milestone-paragraph.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=590)
class MilestoneParagraphAddMenu(BaseParagraphAddMenu):
    """Milestone paragraph add menu"""
    
    label = _("Milestones...")
    label_css_class = 'fa fa-fw fa-arrows-h'
    url = 'add-milestone-paragraph.html'
    paragraph_type = MILESTONE_PARAGRAPH_TYPE
    
    
@pagelet_config(name='add-milestone-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class MilestoneParagraphAddForm(AdminDialogAddForm):
    """Milestone paragraph add form"""
    
    legend = _("Add new milestone paragraph")
    icon_css_class = 'fa fa-fw fa-arrows-h'
    
    fields = field.Fields(IMilestoneParagraph).select('title', 'renderer')
    ajax_handler = 'add-milestone-paragraph.json'
    edit_permission = MANAGE_CONTENT_PERMISSION
    
    def create(self, data):
        return MilestoneParagraph()
    
    def add(self, object):
        IParagraphContainer(self.context).append(object)
        
        
@view_config(name='add-milestone-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class MilestoneParagraphAJAXAddForm(BaseParagraphAJAXAddForm, MilestoneParagraphAddForm):
    """Milestone paragraph add form, JSON renderer"""
    
    
@pagelet_config(name='properties.html', context=IMilestoneParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class MilestoneParagraphPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Milestone paragraph properties edit form"""

    @property
    def title(self):
        content = get_parent(self.context, IWfSharedContent)
        return II18n(content).query_attribute('title', request=self.request)

    legend = _("Edit milestone paragraph properties")
    icon_css_class = 'fa fa-fw fa-arrows-h'

    fields = field.Fields(IMilestoneParagraph).select('title', 'renderer')
    fields['renderer'].widgetFactory = RendererFieldWidget

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION


@view_config(name='properties.json', context=IMilestoneParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class MilestoneParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, MilestoneParagraphPropertiesEditForm):
    """Milestone paragraph properties edit form, JSON renderer"""


@adapter_config(context=(IMilestoneParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm, IMilestonesParentForm)
class MilestoneParagraphInnerEditForm(MilestoneParagraphPropertiesEditForm):
    """Milestone paragraph inner edit form"""

    legend = None
    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=IMilestoneParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class MilestoneParagraphInnerAJAXEditForm(BaseParagraphAJAXEditForm, MilestoneParagraphInnerEditForm):
    """Milestones paragraph inner edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(MilestoneParagraphInnerAJAXEditForm, self).get_ajax_output(changes)
        updated = changes.get(IMilestoneParagraph, ())
        if 'renderer' in updated:
            form = MilestoneParagraphInnerEditForm(self.context, self.request)
            form.update()
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'MyAMS.skin.refreshWidget',
                    'parent_id': '{0}_{1}_{2}'.format(
                        self.context.__class__.__name__,
                        getattr(form.getContent(), '__name__', 'noname').replace('++', ''),
                        form.id),
                    'widget_name': form.widgets['renderer'].name,
                    'content': form.widgets['renderer'].render()
                }
            })
        return output


#
# Milestone paragraph renderer
#

@adapter_config(context=(IMilestoneParagraph, IPyAMSLayer), provides=IParagraphRenderer)
class MilestoneParagraphRenderer(BaseRenderedContentRenderer):
    """Milestone paragraph renderer"""


#
# Milestone items table view
#

class MilestonesTable(ProtectedFormObjectMixin, BaseTable):
    """Milestones view inner table"""

    @property
    def id(self):
        return 'milestones_{0}_list'.format(self.context.__name__)

    hide_header = True
    sortOn = None

    @property
    def cssClasses(self):
        classes = ['table', 'table-bordered', 'table-striped', 'table-hover', 'table-tight']
        permission = self.permission
        if (not permission) or self.request.has_permission(permission, self.context):
            classes.append('table-dnd')
        return {'table': ' '.join(classes)}

    @property
    def data_attributes(self):
        attributes = super(MilestonesTable, self).data_attributes
        attributes['table'] = {
            'id': self.id,
            'data-ams-plugins': 'pyams_content',
            'data-ams-plugin-pyams_content-src': get_resource_path(pyams_content),
            'data-ams-location': absolute_url(IMilestoneContainer(self.context), self.request),
            'data-ams-tablednd-drag-handle': 'td.sorter',
            'data-ams-tablednd-drop-target': 'set-milestones-order.json'
        }
        attributes.setdefault('tr', {}).update({
            'id': lambda x, col: 'milestone_{0}::{1}'.format(get_parent(x, IMilestoneContainerTarget).__name__,
                                                             x.__name__),
            'data-ams-delete-target': 'delete-milestone.json'
        })
        return attributes

    @reify
    def values(self):
        return list(super(MilestonesTable, self).values)


@adapter_config(context=(IMilestoneContainerTarget, IPyAMSLayer, MilestonesTable), provides=IValues)
class MilestonesTableValuesAdapter(ContextRequestViewAdapter):
    """Milestones table values adapter"""

    @property
    def values(self):
        return IMilestoneContainer(self.context).values()


@adapter_config(name='sorter', context=(IMilestoneContainerTarget, IPyAMSLayer, MilestonesTable), provides=IColumn)
class MilestonesTableSorterColumn(ProtectedFormObjectMixin, SorterColumn):
    """Milestones table sorter column"""


@view_config(name='set-milestones-order.json', context=IMilestoneContainer, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
def set_milestones_order(request):
    """Update milestones order"""
    order = list(map(str, json.loads(request.params.get('names'))))
    request.context.updateOrder(order)
    return {'status': 'success'}


@adapter_config(name='show-hide', context=(IMilestoneContainerTarget, IPyAMSLayer, MilestonesTable),
                provides=IColumn)
class MilestonesTableShowHideColumn(ProtectedFormObjectMixin, JsActionColumn):
    """Milestones container visibility switcher column"""

    cssClasses = {'th': 'action',
                  'td': 'action switcher'}

    icon_class = 'fa fa-fw fa-eye'
    icon_hint = _("Switch milestone visibility")

    url = 'PyAMS_content.milestones.switchVisibility'

    weight = 5

    def get_icon(self, item):
        if item.visible:
            icon_class = 'fa fa-fw fa-eye'
        else:
            icon_class = 'fa fa-fw fa-eye-slash text-danger'
        return '<i class="{icon_class}"></i>'.format(icon_class=icon_class)

    def renderCell(self, item):
        if self.permission and not self.request.has_permission(self.permission, context=item):
            return self.get_icon(item)
        else:
            return super(MilestonesTableShowHideColumn, self).renderCell(item)


@view_config(name='set-milestone-visibility.json', context=IMilestoneContainer, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
def set_milestone_visibility(request):
    """Set milestone visibility"""
    container = IMilestoneContainer(request.context)
    milestone = container.get(str(request.params.get('object_name')))
    if milestone is None:
        raise NotFound()
    milestone.visible = not milestone.visible
    return {'visible': milestone.visible}


@adapter_config(name='name', context=(IMilestoneContainerTarget, IPyAMSLayer, MilestonesTable), provides=IColumn)
class MilestonesTableNameColumn(I18nColumn, I18nAttrColumn):
    """Milestones table name column"""

    _header = _("Title")
    attrName = 'title'
    weight = 10


@adapter_config(name='info', context=(IMilestoneContainerTarget, IPyAMSLayer, MilestonesTable), provides=IColumn)
class MilestonesTableInfoColumn(I18nColumn, I18nAttrColumn):
    """Milestones table information column"""

    _header = _("Associated label")
    attrName = 'label'
    weight = 20


@adapter_config(name='anchor', context=(IMilestoneContainerTarget, IPyAMSLayer, MilestonesTable), provides=IColumn)
class MilestonesTableAnchorColumn(I18nColumn, GetAttrColumn):
    """Milestones table anchor column"""

    _header = _("Anchor")
    weight = 30

    def getValue(self, obj):
        if not obj.anchor:
            return '--'
        target = get_parent(self.context, IParagraphContainerTarget)
        if target is None:
            return '--'
        paragraph= IParagraphContainer(target).get(obj.anchor)
        if paragraph is None:
            return '--'
        return II18n(paragraph).query_attribute('title', request=self.request) or '--'


@adapter_config(name='trash', context=(IMilestoneContainerTarget, IPyAMSLayer, MilestonesTable), provides=IColumn)
class MilestonesTableTrashColumn(ProtectedFormObjectMixin, TrashColumn):
    """Milestones table trash column"""


@view_config(name='delete-milestone.json', context=IMilestoneContainer, request_type=IPyAMSLayer,
             permission=MANAGE_PERMISSION, renderer='json', xhr=True)
def delete_milestone(request):
    """Delete milestone"""
    translate = request.localizer.translate
    name = request.params.get('object_name')
    if not name:
        return {
            'status': 'message',
            'messagebox': {
                'status': 'error',
                'content': translate(_("No provided object_name argument!"))
            }
        }
    if name not in request.context:
        return {
            'status': 'message',
            'messagebox': {
                'status': 'error',
                'content': translate(_("Given association name doesn't exist!"))
            }
        }
    del request.context[name]
    return {'status': 'success'}


@adapter_config(name='milestones', context=(IMilestoneContainerTarget, IPyAMSLayer, IMilestonesParentForm),
                provides=IInnerSubForm)
@template_config(template='templates/milestones.pt', layer=IPyAMSLayer)
@implementer(IMilestonesView)
class MilestonesView(InnerAdminDisplayForm):
    """Milestones view"""

    fields = field.Fields(Interface)
    weight = 100

    def __init__(self, context, request, view):
        super(MilestonesView, self).__init__(context, request, view)
        self.table = MilestonesTable(context, request)
        self.table.view = self

    def update(self):
        super(MilestonesView, self).update()
        self.table.update()


#
# Milestones forms
#

@viewlet_config(name='add-milestone.action', context=IMilestoneContainerTarget, layer=IPyAMSLayer, view=IMilestonesView,
                manager=IWidgetTitleViewletManager, permission=MANAGE_CONTENT_PERMISSION, weight=1)
class MilestoneAddAction(ToolbarAction):
    """Milestone add action"""

    label = _("Add milestone")
    label_css_class = 'fa fa-fw fa-plus'
    url = 'add-milestone.html'
    modal_target = True


@pagelet_config(name='add-milestone.html', context=IMilestoneContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class MilestoneAddForm(AdminDialogAddForm):
    """Milestone add form"""

    legend = _("Add new milestone")
    icon_css_class = 'fa fa-fw fa-arrow-h'

    fields = field.Fields(IMilestone).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'add-milestone.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def create(self, data):
        return Milestone()

    def add(self, object):
        IMilestoneContainer(self.context).append(object)


@view_config(name='add-milestone.json', context=IMilestoneContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class MilestoneAJAXAddForm(AJAXAddForm, MilestoneAddForm):
    """Milestone add form, JSON renderer"""

    def get_ajax_output(self, changes):
        table = MilestonesTable(self.context, self.request)
        table.update()
        return {
            'status': 'success',
            'message': self.request.localizer.translate(_("Milestone was correctly added")),
            'events': [{
                'event': 'myams.refresh',
                'options': {
                    'handler': 'PyAMS_content.milestones.refreshMilestones',
                    'object_id': table.id,
                    'table': table.render()
                }
            }]
        }


@pagelet_config(name='properties.html', context=IMilestone, layer=IPyAMSLayer, permission=MANAGE_CONTENT_PERMISSION)
class MilestonePropertiesEditForm(AdminDialogEditForm):
    """Milestone properties edit form"""

    legend = _("Edit milestone properties")
    icon_css_class = 'fa fa-fw fa-arrows-h'

    fields = field.Fields(IMilestone).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION


@view_config(name='properties.json', context=IMilestone, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class MilestonePropertiesAJAXEditForm(AJAXEditForm, MilestonePropertiesEditForm):
    """Milestone properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(MilestonePropertiesAJAXEditForm, self).get_ajax_output(changes)
        updated = changes.get(IMilestone, ())
        if ('title' in updated) or ('anchor' in updated):
            target = get_parent(self.context, IMilestoneContainerTarget)
            table = MilestonesTable(target, self.request)
            table.update()
            row = table.setUpRow(self.context)
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'MyAMS.skin.refreshRow',
                    'object_id': 'milestone_{0}::{1}'.format(target.__name__,
                                                             self.context.__name__),
                    'row': table.renderRow(row)
                }
            })
        return output
