#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

# import standard library
import sys

from random import randint
from uuid import uuid4

# import interfaces
from pyams_content.interfaces import MANAGE_SITE_ROOT_PERMISSION, IBaseContent
from pyams_content.reference.pictograms.interfaces import IPictogramTable, IPictogram
from pyams_content.reference.zmi.table import ReferenceTableContentsTable, ReferenceTableContentsView
from pyams_form.form import AJAXAddForm, AJAXEditForm
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_zmi.layer import IAdminLayer
from z3c.table.interfaces import IColumn

# import packages
from pyams_content.reference.pictograms import Pictogram
from pyams_i18n.column import I18nAttrColumn
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.table import I18nColumn
from pyams_skin.viewlet.toolbar import ToolbarAction
from pyams_utils.adapter import adapter_config
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field
from z3c.table.column import GetAttrColumn

from pyams_content import _


@viewlet_config(name='add-pictogram.action', context=IPictogramTable, layer=IPyAMSLayer,
                manager=IWidgetTitleViewletManager, view=ReferenceTableContentsTable,
                permission=MANAGE_SITE_ROOT_PERMISSION)
class PictogramAddAction(ToolbarAction):
    """Pictogram add action"""

    label = _("Add pictogram")

    url = 'add-pictogram.html'
    modal_target = True


@pagelet_config(name='add-pictogram.html', context=IPictogramTable, layer=IPyAMSLayer,
                permission=MANAGE_SITE_ROOT_PERMISSION)
class PictogramAddForm(AdminDialogAddForm):
    """Pictogram add form"""

    legend = _("Add new pictogram")

    fields = field.Fields(IPictogram).omit('__parent__', '__name__')
    ajax_handler = 'add-pictogram.json'
    edit_permission = MANAGE_SITE_ROOT_PERMISSION

    def create(self, data):
        return Pictogram()

    def add(self, object):
        name = str(uuid4())
        self.context[name] = object

    def nextURL(self):
        return 'contents.html'


@view_config(name='add-pictogram.json', context=IPictogramTable, request_type=IPyAMSLayer,
             permission=MANAGE_SITE_ROOT_PERMISSION, renderer='json', xhr=True)
class PictogramAJAXAddForm(AJAXAddForm, PictogramAddForm):
    """Pictogram add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IPictogram, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class PictogramEditForm(AdminDialogEditForm):
    """Pictogram properties edit form"""

    legend = _("Edit pictogram properties")
    dialog_class = 'modal-large'

    fields = field.Fields(IPictogram).omit('__parent__', '__name__')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_SITE_ROOT_PERMISSION


@view_config(name='properties.json', context=IPictogram, request_type=IPyAMSLayer,
             permission=MANAGE_SITE_ROOT_PERMISSION, renderer='json', xhr=True)
class PictogramAJAXEditForm(AJAXEditForm, PictogramEditForm):
    """Pictogram edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(PictogramAJAXEditForm, self).get_ajax_output(changes)
        if 'image' in changes.get(IPictogram, ()):
            image = II18n(self.context).query_attribute('image', request=self.request)
            timestamp = randint(0, sys.maxsize)
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'MyAMS.skin.refreshRowCell',
                    'object_id': '{0}::{1}'.format(ReferenceTableContentsTable.id, self.context.__name__),
                    'col_name': 'image',
                    'cell': '<img src="{0}?_={1}" />'.format(absolute_url(image, self.request, '++thumb++32x32'),
                                                             timestamp)
                }
            })
        if 'title' in changes.get(IBaseContent, ()):
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'MyAMS.skin.refreshRowCell',
                    'object_id': '{0}::{1}'.format(ReferenceTableContentsTable.id, self.context.__name__),
                    'col_name': 'name',
                    'cell': II18n(self.context).query_attribute('title', request=self.request)
                }
            })
        return output


#
# Pictogram table
#

class PictogramTableContentsTable(ReferenceTableContentsTable):
    """Pictograms table contents table"""

    @property
    def data_attributes(self):
        attributes = super(PictogramTableContentsTable, self).data_attributes
        attributes.setdefault('table', {}).update({
            'data-ams-datatable-sorting': '1,asc'
        })
        return attributes


@adapter_config(name='image', context=(IPictogramTable, IAdminLayer, PictogramTableContentsTable), provides=IColumn)
class PictogramTableImageColumn(GetAttrColumn):
    """Pictogram table image column"""

    header = ''
    weight = 1

    cssClasses = {'td': 'text-center width-50'}
    dt_sortable = 'false'

    def getValue(self, obj):
        image = II18n(obj).query_attribute('image', request=self.request)
        if image:
            return '<img src="{0}" />'.format(absolute_url(image, self.request, '++thumb++32x32'))


@adapter_config(name='name', context=(IPictogramTable, IAdminLayer, PictogramTableContentsTable), provides=IColumn)
class PictogramTableNameColumn(I18nColumn, I18nAttrColumn):
    """Pictogram table name column"""

    _header = _("Title")
    weight = 10
    attrName = 'title'


@pagelet_config(name='contents.html', context=IPictogramTable, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class PictogramTableContentsView(ReferenceTableContentsView):
    """Pictograms table contents view"""

    table_class = PictogramTableContentsTable
